@@ -81,9 +81,11 @@
 #ifdef HAVE_NET_IF_DL_H
 #include <net/if_dl.h>
 #endif
+
 #ifdef HAVE_NET_IF_TYPES_H
 #include <net/if_types.h>
 #endif
+
 #if defined(HAVE_NET_IF_ARP_H) && !defined(ARPHRD_ETHER)
 #include <net/if_arp.h>
 #endif				/* defined(HAVE_NET_IF_ARP_H) && !defined(ARPHRD_ETHER) */
@@ -100,3 +102,7 @@
 #include <ifaddrs.h>
 #endif
 
+#ifdef HAVE_LINUX_IF_ARP_H
+#include <linux/if_arp.h>
+#endif
+
