@@ -46,6 +46,7 @@ void iface_init_defaults(struct Interface *iface)
 	iface->mipv6.HomeAgentLifetime = -1;
 
 	iface->AdvLinkMTU = DFLT_AdvLinkMTU;
+	iface->AdvRAMTU = DFLT_AdvRAMTU;
 
 }
 
@@ -90,6 +91,12 @@ int setup_iface(int sock, struct Interface *iface)
 		return -1;
 	}
 
+	/* Check if we a usable RA source address */
+	if(iface->props.if_addr_rasrc == NULL) {
+		dlog(LOG_DEBUG, 5, "no configured AdvRASrcAddress present, skipping send");
+		return -1;
+	}
+
 	/* join the allrouters multicast group so we get the solicitations */
 	if (setup_allrouters_membership(sock, iface) < 0) {
 		return -1;
@@ -148,6 +155,8 @@ int check_iface(struct Interface *iface)
 {
 	int res = 0;
 	int MIPv6 = 0;
+	struct in6_addr zeroaddr;
+	memset(&zeroaddr, 0, sizeof(zeroaddr));
 
 	/* Check if we use Mobile IPv6 extensions */
 	if (iface->ra_header_info.AdvHomeAgentFlag || iface->mipv6.AdvHomeAgentInfo || iface->mipv6.AdvIntervalOpt) {
@@ -264,6 +273,27 @@ int check_iface(struct Interface *iface)
 			res = -1;
 		}
 
+		/* For the default route 0::/0, we need to explicitly check the
+		 * lifetime against the AdvDefaultLifetime value.
+		 *
+		 * If exactly one of the two has a zero value, then nodes processing
+		 * the RA may have a flap in their default route.
+		 *
+		 * AdvDefaultLifetime == 0 && route.AdvRouteLifetime > 0:
+		 * - default route is deleted and then re-added.
+		 * AdvDefaultLifetime > 0 && route.AdvRouteLifetime == 0:
+		 * - default route is added and then deleted.
+		 */
+		if(IN6_IS_ADDR_UNSPECIFIED(&(route->Prefix))) {
+			int route_zerolife = (route->AdvRouteLifetime == 0);
+			int defaultroute_zerolife = (iface->ra_header_info.AdvDefaultLifetime == 0);
+			if( route_zerolife ^ defaultroute_zerolife ) {
+				flog(LOG_ERR, "route 0::/0 lifetime (%u) conflicts with AdvDefaultLifetime (%u), default routes will flap!",
+						route->AdvRouteLifetime, iface->ra_header_info.AdvDefaultLifetime);
+				// res = -1; // In some future version, abort on this configuration error.
+			}
+		}
+
 		route = route->next;
 	}
 
