@@ -13,8 +13,7 @@
  *
  */
 
-#ifndef PATHNAMES_H
-#define PATHNAMES_H
+#pragma once
 
 #ifndef PATH_RADVD_CONF
 #define PATH_RADVD_CONF "/etc/radvd.conf"
@@ -28,12 +27,13 @@
 #define PATH_RADVD_LOG "/var/log/radvd.log"
 #endif
 
-#define PATH_PROC_NET_IF_INET6 "/proc/net/if_inet6"
 #define PATH_PROC_NET_IGMP6 "/proc/net/igmp6"
 
 #ifdef __linux__
 #define SYSCTL_IP6_FORWARDING CTL_NET, NET_IPV6, NET_IPV6_CONF, NET_PROTO_CONF_ALL, NET_IPV6_FORWARDING
+#define SYSCTL_IP6_AUTOCONFIG CTL_NET, NET_IPV6, NET_IPV6_CONF, NET_PROTO_CONF_ALL, NET_IPV6_AUTOCONF
 #define PROC_SYS_IP6_FORWARDING "/proc/sys/net/ipv6/conf/all/forwarding"
+#define PROC_SYS_IP6_AUTOCONFIG "/proc/sys/net/ipv6/conf/%s/autoconf"
 #define PROC_SYS_IP6_LINKMTU "/proc/sys/net/ipv6/conf/%s/mtu"
 #define PROC_SYS_IP6_CURHLIM "/proc/sys/net/ipv6/conf/%s/hop_limit"
 #define PROC_SYS_IP6_BASEREACHTIME_MS "/proc/sys/net/ipv6/neigh/%s/base_reachable_time_ms"
@@ -43,5 +43,3 @@
 #else				/* BSD */
 #define SYSCTL_IP6_FORWARDING CTL_NET, PF_INET6, IPPROTO_IPV6, IPV6CTL_FORWARDING
 #endif
-
-#endif
