@@ -108,7 +108,11 @@ void process(int sock, struct Interface *interfaces, unsigned char *msg, int len
 		dlog(LOG_DEBUG, 3, "%s received RS from: %s", if_name, addr_str);
 		process_rs(sock, iface, msg, len, addr);
 	} else if (icmph->icmp6_type == ND_ROUTER_ADVERT) {
-		dlog(LOG_DEBUG, 3, "%s received RA from: %s", if_name, addr_str);
+		if (0 == memcmp(&addr->sin6_addr, &iface->props.if_addr, sizeof(iface->props.if_addr))) {
+			dlog(LOG_DEBUG, 3, "%s received RA from: %s (myself)", if_name, addr_str);
+		} else {
+			dlog(LOG_DEBUG, 3, "%s received RA from: %s", if_name, addr_str);
+		}
 		process_ra(iface, msg, len, addr);
 	}
 }
@@ -251,7 +255,7 @@ static void process_ra(struct Interface *iface, unsigned char *msg, int len, str
 				struct AdvPrefix *prefix = iface->AdvPrefixList;
 				while (prefix) {
 					char prefix_str[INET6_ADDRSTRLEN];
-					if (prefix->enabled && (prefix->PrefixLen == pinfo->nd_opt_pi_prefix_len)
+					if ((prefix->PrefixLen == pinfo->nd_opt_pi_prefix_len)
 					    && addr_match(&prefix->Prefix, &pinfo->nd_opt_pi_prefix, prefix->PrefixLen)) {
 						addrtostr(&prefix->Prefix, prefix_str, sizeof(prefix_str));
 
