@@ -19,25 +19,43 @@
 
 static int really_send(int sock, struct in6_addr const *dest, struct properties const *props, struct safe_buffer const *sb);
 static int send_ra(int sock, struct Interface *iface, struct in6_addr const *dest);
-static void build_ra(struct safe_buffer * sb, struct Interface const * iface);
+static struct safe_buffer_list * build_ra_options(struct Interface const * iface, struct in6_addr const *dest);
 
 static int ensure_iface_setup(int sock, struct Interface *iface);
 static void decrement_lifetime(const time_t secs, uint32_t * lifetime);
 static void update_iface_times(struct Interface * iface);
 
-static void add_ra_header(struct safe_buffer * sb, struct ra_header_info const * ra_header_info, int cease_adv);
-static void add_prefixs(struct safe_buffer * sb, char const * ifname, struct AdvPrefix const * prefix, int cease_adv);
-static void add_one_prefix(struct safe_buffer * sb, char const * ifname, struct AdvPrefix const * prefix, int cease_adv);
-static void add_route(struct safe_buffer * sb, struct AdvRoute const *route, int cease_adv);
-static void add_rdnss(struct safe_buffer * sb, struct AdvRDNSS const *rdnss, int cease_adv);
+// Option helpers
 static size_t serialize_domain_names(struct safe_buffer * safe_buffer, struct AdvDNSSL const *dnssl);
-static void add_dnssl(struct safe_buffer * sb, struct AdvDNSSL const *dnssl, int cease_adv);
-static void add_mtu(struct safe_buffer * sb, uint32_t AdvLinkMTU);
-static void add_sllao(struct safe_buffer * sb, struct sllao const *sllao);
-static void add_mipv6_rtr_adv_interval(struct safe_buffer * sb, double MaxRtrAdvInterval);
-static void add_mipv6_home_agent_info(struct safe_buffer * sb, struct mipv6 const * mipv6);
-static void add_lowpanco(struct safe_buffer * sb, struct AdvLowpanCo const *lowpanco);
-static void add_abro(struct safe_buffer * sb, struct AdvAbro const *abroo);
+
+// Options that only need a single block
+static void add_ra_header(struct safe_buffer * sb, struct ra_header_info const * ra_header_info, int cease_adv);
+static void add_ra_option_prefix(struct safe_buffer * sb, struct AdvPrefix const * prefix, int cease_adv);
+static void add_ra_option_mtu(struct safe_buffer * sb, uint32_t AdvLinkMTU);
+static void add_ra_option_sllao(struct safe_buffer * sb, struct sllao const *sllao);
+static void add_ra_option_mipv6_rtr_adv_interval(struct safe_buffer * sb, double MaxRtrAdvInterval);
+static void add_ra_option_mipv6_home_agent_info(struct safe_buffer * sb, struct mipv6 const * mipv6);
+static void add_ra_option_lowpanco(struct safe_buffer * sb, struct AdvLowpanCo const *lowpanco);
+static void add_ra_option_abro(struct safe_buffer * sb, struct AdvAbro const *abroo);
+
+// Options that generate 0 or more blocks
+static struct safe_buffer_list* add_ra_options_prefix(struct safe_buffer_list * sbl, struct Interface const * iface, char const * ifname, struct AdvPrefix const * prefix, int cease_adv, struct in6_addr const *dest);
+static struct safe_buffer_list* add_ra_options_route(struct safe_buffer_list * sbl, struct Interface const * iface, struct AdvRoute const *route, int cease_adv, struct in6_addr const *dest);
+static struct safe_buffer_list* add_ra_options_rdnss(struct safe_buffer_list * sbl, struct Interface const * iface, struct AdvRDNSS const *rdnss, int cease_adv, struct in6_addr const *dest);
+static struct safe_buffer_list* add_ra_options_dnssl(struct safe_buffer_list * sbl, struct Interface const * iface, struct AdvDNSSL const *dnssl, int cease_adv, struct in6_addr const *dest);
+
+// Scheduling of options per RFC7772
+static int schedule_helper(struct in6_addr const *dest, struct Interface const *iface, int option_lifetime);
+static int schedule_option_prefix(struct in6_addr const *dest, struct Interface const *iface, struct AdvPrefix const *prefix);
+static int schedule_option_route(struct in6_addr const *dest, struct Interface const *iface, struct AdvRoute const *route);
+static int schedule_option_rdnss(struct in6_addr const *dest, struct Interface const *iface, struct AdvRDNSS const *rdnss);
+static int schedule_option_dnssl(struct in6_addr const *dest, struct Interface const *iface, struct AdvDNSSL const *dnssl);
+static int schedule_option_mtu(struct in6_addr const *dest, struct Interface const *iface);
+static int schedule_option_sllao(struct in6_addr const *dest, struct Interface const *iface);
+static int schedule_option_mipv6_rtr_adv_interval(struct in6_addr const *dest, struct Interface const *iface);
+static int schedule_option_mipv6_home_agent_info(struct in6_addr const *dest, struct Interface const *iface);
+static int schedule_option_lowpanco(struct in6_addr const *dest, struct Interface const *iface);
+static int schedule_option_abro(struct in6_addr const *dest, struct Interface const *iface);
 
 #ifdef UNIT_TEST
 #include "test/send.c"
@@ -91,7 +109,7 @@ int send_ra_forall(int sock, struct Interface *iface, struct in6_addr *dest)
 	/* If we refused a client's solicitation, log it if debugging is high enough */
 	if (get_debuglevel() >= 5) {
 		char address_text[INET6_ADDRSTRLEN] = { "" };
-		inet_ntop(AF_INET6, dest, address_text, INET6_ADDRSTRLEN);
+		addrtostr(dest, address_text, INET6_ADDRSTRLEN);
 		dlog(LOG_DEBUG, 5, "Not answering request from %s, not configured", address_text);
 	}
 
@@ -192,7 +210,7 @@ static void add_ra_header(struct safe_buffer * sb, struct ra_header_info const *
 	safe_buffer_append(sb, &radvert, sizeof(radvert));
 }
 
-static void add_one_prefix(struct safe_buffer * sb, char const * ifname, struct AdvPrefix const * prefix, int cease_adv)
+static void add_ra_option_prefix(struct safe_buffer * sb, struct AdvPrefix const * prefix, int cease_adv)
 {
 	struct nd_opt_prefix_info pinfo;
 
@@ -224,7 +242,8 @@ static void add_one_prefix(struct safe_buffer * sb, char const * ifname, struct
 
 	safe_buffer_append(sb, &pinfo, sizeof(pinfo));
 }
-static void add_auto_prefixes_6to4(struct safe_buffer * sb, char const * ifname, struct AdvPrefix const *prefix, int cease_adv)
+
+static struct safe_buffer_list * add_auto_prefixes_6to4(struct safe_buffer_list * sbl, struct Interface const * iface, char const * ifname, struct AdvPrefix const *prefix, int cease_adv, struct in6_addr const *dest)
 {
 #ifdef HAVE_IFADDRS_H
 	struct AdvPrefix xprefix = *prefix;
@@ -245,13 +264,17 @@ static void add_auto_prefixes_6to4(struct safe_buffer * sb, char const * ifname,
 		(void)xprefix.curr_validlft;
 		(void)xprefix.curr_preferredlft;
 
-		add_one_prefix(sb, ifname, &xprefix, cease_adv);
+		if(cease_adv || schedule_option_prefix(dest, iface, &xprefix)) {
+			sbl = safe_buffer_list_append(sbl);
+			add_ra_option_prefix(sbl->sb, &xprefix, cease_adv);
+		}
 	}
 #endif
+	return sbl;
 }
 
 
-static void add_auto_prefixes(struct safe_buffer * sb, char const * ifname, struct AdvPrefix const *prefix, int cease_adv)
+static struct safe_buffer_list * add_auto_prefixes(struct safe_buffer_list * sbl, struct Interface const * iface, char const * ifname, struct AdvPrefix const *prefix, int cease_adv, struct in6_addr const *dest)
 {
 #ifdef HAVE_IFADDRS_H
 	struct AdvPrefix xprefix;
@@ -287,15 +310,19 @@ static void add_auto_prefixes(struct safe_buffer * sb, char const * ifname, stru
 		(void)xprefix.curr_validlft;
 		(void)xprefix.curr_preferredlft;
 
-		add_one_prefix(sb, ifname, &xprefix, cease_adv);
+		if(cease_adv || schedule_option_prefix(dest, iface, &xprefix)) {
+			sbl = safe_buffer_list_append(sbl);
+			add_ra_option_prefix(sbl->sb, &xprefix, cease_adv);
+		}
 	}
 
 	if (ifap)
 		freeifaddrs(ifap);
 #endif
+	return sbl;
 }
 
-static void add_prefixs(struct safe_buffer * sb, char const * ifname, struct AdvPrefix const *prefix, int cease_adv)
+static struct safe_buffer_list* add_ra_options_prefix(struct safe_buffer_list * sbl, struct Interface const * iface, char const * ifname, struct AdvPrefix const *prefix, int cease_adv, struct in6_addr const *dest)
 {
 	while (prefix) {
 		if ((!prefix->DecrementLifetimesFlag || prefix->curr_preferredlft > 0)) {
@@ -303,23 +330,27 @@ static void add_prefixs(struct safe_buffer * sb, char const * ifname, struct Adv
 			if (prefix->if6to4[0] || prefix->if6[0] || 0 == memcmp(&prefix->Prefix, &zero, sizeof(zero))) {
 				if (prefix->if6to4[0]) {
 					dlog(LOG_DEBUG, 4, "if6to4 auto prefix detected on iface %s", ifname);
-					add_auto_prefixes_6to4(sb, prefix->if6to4, prefix, cease_adv);
+					sbl = add_auto_prefixes_6to4(sbl, iface, prefix->if6to4, prefix, cease_adv, dest);
 				}
 				if (prefix->if6[0]) {
 					dlog(LOG_DEBUG, 4, "if6 auto prefix detected on iface %s", ifname);
-					add_auto_prefixes(sb, prefix->if6, prefix, cease_adv);
+					sbl = add_auto_prefixes(sbl, iface, prefix->if6, prefix, cease_adv, dest);
 				}
 				if (0 == memcmp(&prefix->Prefix, &zero, sizeof(zero))) {
 					dlog(LOG_DEBUG, 4, "::/64 auto prefix detected on iface %s", ifname);
-					add_auto_prefixes(sb, ifname, prefix, cease_adv);
+					sbl = add_auto_prefixes(sbl, iface, iface->props.name, prefix, cease_adv, dest);
 				}
 			} else {
-				add_one_prefix(sb, ifname, prefix, cease_adv);
+				if(cease_adv || schedule_option_prefix(dest, iface, prefix)) {
+					sbl = safe_buffer_list_append(sbl);
+					add_ra_option_prefix(sbl->sb, prefix, cease_adv);
+				}
 			}
 		}
 
 		prefix = prefix->next;
 	}
+	return sbl;
 }
 
 
@@ -359,6 +390,8 @@ static size_t serialize_domain_names(struct safe_buffer * safe_buffer, struct Ad
 			else
 				label_len = (unsigned char)(strchr(label, '.') - label);
 
+			// +8 is for null & padding, only allocate once.
+			safe_buffer_resize(safe_buffer, safe_buffer->used + sizeof(label_len) + label_len + 8);
 			len += safe_buffer_append(safe_buffer, &label_len, sizeof(label_len));
 			len += safe_buffer_append(safe_buffer, label, label_len);
 
@@ -377,11 +410,16 @@ static size_t serialize_domain_names(struct safe_buffer * safe_buffer, struct Ad
 	return len;
 }
 
-static void add_route(struct safe_buffer * sb, struct AdvRoute const *route, int cease_adv)
+static struct safe_buffer_list* add_ra_options_route(struct safe_buffer_list *sbl, struct Interface const * iface, struct AdvRoute const *route, int cease_adv, struct in6_addr const *dest)
 {
 	while (route) {
 		struct nd_opt_route_info_local rinfo;
 
+		if(!cease_adv && !schedule_option_route(dest, iface, route)) {
+			route = route->next;
+			continue;
+		}
+
 		memset(&rinfo, 0, sizeof(rinfo));
 
 		rinfo.nd_opt_ri_type = ND_OPT_ROUTE_INFORMATION;
@@ -398,17 +436,24 @@ static void add_route(struct safe_buffer * sb, struct AdvRoute const *route, int
 
 		memcpy(&rinfo.nd_opt_ri_prefix, &route->Prefix, sizeof(struct in6_addr));
 
-		safe_buffer_append(sb, &rinfo, sizeof(rinfo));
+		sbl = safe_buffer_list_append(sbl);
+		safe_buffer_append(sbl->sb, &rinfo, sizeof(rinfo));
 
 		route = route->next;
 	}
+	return sbl;
 }
 
-static void add_rdnss(struct safe_buffer * sb, struct AdvRDNSS const *rdnss, int cease_adv)
+static struct safe_buffer_list * add_ra_options_rdnss(struct safe_buffer_list * sbl, struct Interface const * iface, struct AdvRDNSS const *rdnss, int cease_adv, struct in6_addr const *dest)
 {
 	while (rdnss) {
 		struct nd_opt_rdnss_info_local rdnssinfo;
 
+		if(!cease_adv && !schedule_option_rdnss(dest, iface, rdnss)) {
+			rdnss = rdnss->next;
+			continue;
+		}
+
 		memset(&rdnssinfo, 0, sizeof(rdnssinfo));
 
 		rdnssinfo.nd_opt_rdnssi_type = ND_OPT_RDNSS_INFORMATION;
@@ -425,14 +470,18 @@ static void add_rdnss(struct safe_buffer * sb, struct AdvRDNSS const *rdnss, int
 		memcpy(&rdnssinfo.nd_opt_rdnssi_addr2, &rdnss->AdvRDNSSAddr2, sizeof(struct in6_addr));
 		memcpy(&rdnssinfo.nd_opt_rdnssi_addr3, &rdnss->AdvRDNSSAddr3, sizeof(struct in6_addr));
 
-		safe_buffer_append(sb, &rdnssinfo, sizeof(rdnssinfo) - (3 - rdnss->AdvRDNSSNumber) * sizeof(struct in6_addr));
+		sbl = safe_buffer_list_append(sbl);
+		safe_buffer_append(sbl->sb, &rdnssinfo, sizeof(rdnssinfo) - (3 - rdnss->AdvRDNSSNumber) * sizeof(struct in6_addr));
 
 		rdnss = rdnss->next;
 	}
+
+	return sbl;
 }
 
-static void add_dnssl(struct safe_buffer * safe_buffer, struct AdvDNSSL const *dnssl, int cease_adv)
+static struct safe_buffer_list * add_ra_options_dnssl(struct safe_buffer_list * sbl, struct Interface const * iface, struct AdvDNSSL const *dnssl, int cease_adv, struct in6_addr const *dest)
 {
+	struct safe_buffer *serialized_domains = new_safe_buffer();
 	while (dnssl) {
 
 		/* *INDENT-OFF* */
@@ -493,10 +542,20 @@ static void add_dnssl(struct safe_buffer * safe_buffer, struct AdvDNSSL const *d
 		/* *INDENT-ON* */
 		struct nd_opt_dnssl_info_local dnsslinfo;
 
+		if(!cease_adv && !schedule_option_dnssl(dest, iface, dnssl)) {
+			dnssl = dnssl->next;
+			continue;
+		}
+
 		memset(&dnsslinfo, 0, sizeof(dnsslinfo));
 
-		size_t const domain_name_bytes = serialize_domain_names(0, dnssl);
+		serialized_domains->used = 0;
+		size_t const domain_name_bytes = serialize_domain_names(serialized_domains, dnssl);
 		size_t const bytes = sizeof(dnsslinfo) + domain_name_bytes;
+		if(bytes > (256 * 8)) {
+			flog(LOG_ERR, "DNSSL too long for RA option, must be < 2048 bytes.  Exiting.");
+			exit(1);
+		}
 		
 		dnsslinfo.nd_opt_dnssli_type = ND_OPT_DNSSL_INFORMATION;
 		dnsslinfo.nd_opt_dnssli_len = (bytes + 7) / 8;
@@ -510,20 +569,24 @@ static void add_dnssl(struct safe_buffer * safe_buffer, struct AdvDNSSL const *d
 
 		size_t const padding = dnsslinfo.nd_opt_dnssli_len * 8 - bytes;
 
-		safe_buffer_append(safe_buffer, &dnsslinfo, sizeof(dnsslinfo));
-		serialize_domain_names(safe_buffer, dnssl);
-		safe_buffer_pad(safe_buffer, padding);
+		sbl = safe_buffer_list_append(sbl);
+		safe_buffer_resize(sbl->sb, sbl->sb->used + sizeof(dnsslinfo) + domain_name_bytes + padding);
+		safe_buffer_append(sbl->sb, &dnsslinfo, sizeof(dnsslinfo));
+		safe_buffer_append(sbl->sb, serialized_domains->buffer, serialized_domains->used);
+		safe_buffer_pad(sbl->sb, padding);
+		//abort();
 
 		dnssl = dnssl->next;
 	}
-
+	return sbl;
+	safe_buffer_free(serialized_domains);
 
 }
 
 /*
  * add Source Link-layer Address option
  */
-static void add_sllao(struct safe_buffer * sb, struct sllao const *sllao)
+static void add_ra_option_sllao(struct safe_buffer * sb, struct sllao const *sllao)
 {
 	/* *INDENT-OFF* */
 	/*
@@ -570,7 +633,7 @@ static void add_sllao(struct safe_buffer * sb, struct sllao const *sllao)
 	safe_buffer_pad(sb, sllao_len * 8 - sllao_bytes);
 }
 
-static void add_mtu(struct safe_buffer * sb, uint32_t AdvLinkMTU)
+static void add_ra_option_mtu(struct safe_buffer * sb, uint32_t AdvLinkMTU)
 {
 	struct nd_opt_mtu mtu;
 
@@ -588,7 +651,7 @@ static void add_mtu(struct safe_buffer * sb, uint32_t AdvLinkMTU)
  * Mobile IPv6 ext: Advertisement Interval Option to support
  * movement detection of mobile nodes
  */
-static void add_mipv6_rtr_adv_interval(struct safe_buffer * sb, double MaxRtrAdvInterval)
+static void add_ra_option_mipv6_rtr_adv_interval(struct safe_buffer * sb, double MaxRtrAdvInterval)
 {
 	uint32_t ival = 1000;
 
@@ -614,7 +677,7 @@ static void add_mipv6_rtr_adv_interval(struct safe_buffer * sb, double MaxRtrAdv
  * Mobile IPv6 ext: Home Agent Information Option to support
  * Dynamic Home Agent Address Discovery
  */
-static void add_mipv6_home_agent_info(struct safe_buffer * sb, struct mipv6 const * mipv6)
+static void add_ra_option_mipv6_home_agent_info(struct safe_buffer * sb, struct mipv6 const * mipv6)
 {
 	struct HomeAgentInfo ha_info;
 
@@ -632,7 +695,7 @@ static void add_mipv6_home_agent_info(struct safe_buffer * sb, struct mipv6 cons
 /*
  * Add 6co option
  */
-static void add_lowpanco(struct safe_buffer * sb, struct AdvLowpanCo const *lowpanco)
+static void add_ra_option_lowpanco(struct safe_buffer * sb, struct AdvLowpanCo const *lowpanco)
 {
 	struct nd_opt_6co co;
 
@@ -649,7 +712,7 @@ static void add_lowpanco(struct safe_buffer * sb, struct AdvLowpanCo const *lowp
 	safe_buffer_append(sb, &co, sizeof(co));
 }
 
-static void add_abro(struct safe_buffer * sb, struct AdvAbro const *abroo)
+static void add_ra_option_abro(struct safe_buffer * sb, struct AdvAbro const *abroo)
 {
 	struct nd_opt_abro abro;
 
@@ -666,51 +729,67 @@ static void add_abro(struct safe_buffer * sb, struct AdvAbro const *abroo)
 }
 
 
-static void build_ra(struct safe_buffer * sb, struct Interface const * iface)
+static struct safe_buffer_list * build_ra_options(struct Interface const * iface, struct in6_addr const *dest)
 {
-	add_ra_header(sb, &iface->ra_header_info, iface->state_info.cease_adv);
+	struct safe_buffer_list *sbl = new_safe_buffer_list();
+	struct safe_buffer_list *cur = sbl;
 
 	if (iface->AdvPrefixList) {
-		add_prefixs(sb, iface->props.name, iface->AdvPrefixList, iface->state_info.cease_adv);
+		cur = add_ra_options_prefix(cur, iface, iface->props.name, iface->AdvPrefixList, iface->state_info.cease_adv, dest);
 	}
 
 	if (iface->AdvRouteList) {
-		add_route(sb, iface->AdvRouteList, iface->state_info.cease_adv);
+		cur = add_ra_options_route(cur, iface, iface->AdvRouteList, iface->state_info.cease_adv, dest);
 	}
 
 	if (iface->AdvRDNSSList) {
-		add_rdnss(sb, iface->AdvRDNSSList, iface->state_info.cease_adv);
+		cur = add_ra_options_rdnss(cur, iface, iface->AdvRDNSSList, iface->state_info.cease_adv, dest);
 	}
 
 	if (iface->AdvDNSSLList) {
-		add_dnssl(sb, iface->AdvDNSSLList, iface->state_info.cease_adv);
+		cur = add_ra_options_dnssl(cur, iface, iface->AdvDNSSLList, iface->state_info.cease_adv, dest);
 	}
 
-	if (iface->AdvLinkMTU != 0) {
-		add_mtu(sb, iface->AdvLinkMTU);
+	if (iface->AdvLinkMTU != 0 && schedule_option_mtu(dest, iface)) {
+		cur->next = new_safe_buffer_list();
+		cur = cur->next;
+		add_ra_option_mtu(cur->sb, iface->AdvLinkMTU);
 	}
 
-	if (iface->AdvSourceLLAddress && iface->sllao.if_hwaddr_len > 0) {
-		add_sllao(sb, &iface->sllao);
+	if (iface->AdvSourceLLAddress && iface->sllao.if_hwaddr_len > 0 && schedule_option_sllao(dest, iface)) {
+		cur->next = new_safe_buffer_list();
+		cur = cur->next;
+		add_ra_option_sllao(cur->sb, &iface->sllao);
 	}
 
-	if (iface->mipv6.AdvIntervalOpt) {
-		add_mipv6_rtr_adv_interval(sb, iface->MaxRtrAdvInterval);
+	if (iface->mipv6.AdvIntervalOpt && schedule_option_mipv6_rtr_adv_interval(dest, iface)) {
+		cur->next = new_safe_buffer_list();
+		cur = cur->next;
+		add_ra_option_mipv6_rtr_adv_interval(cur->sb, iface->MaxRtrAdvInterval);
 	}
 
-	if (iface->mipv6.AdvHomeAgentInfo
+	if (iface->mipv6.AdvHomeAgentInfo && schedule_option_mipv6_home_agent_info(dest, iface)
 	    && (iface->mipv6.AdvMobRtrSupportFlag || iface->mipv6.HomeAgentPreference != 0
 		|| iface->mipv6.HomeAgentLifetime != iface->ra_header_info.AdvDefaultLifetime)) {
-		add_mipv6_home_agent_info(sb, &iface->mipv6);
+		cur->next = new_safe_buffer_list();
+		cur = cur->next;
+		add_ra_option_mipv6_home_agent_info(cur->sb, &iface->mipv6);
 	}
 
-	if (iface->AdvLowpanCoList) {
-		add_lowpanco(sb, iface->AdvLowpanCoList);
+	if (iface->AdvLowpanCoList && schedule_option_lowpanco(dest, iface)) {
+		cur->next = new_safe_buffer_list();
+		cur = cur->next;
+		add_ra_option_lowpanco(cur->sb, iface->AdvLowpanCoList);
 	}
 
-	if (iface->AdvAbroList) {
-		add_abro(sb, iface->AdvAbroList);
+	if (iface->AdvAbroList && schedule_option_abro(dest, iface)) {
+		cur->next = new_safe_buffer_list();
+		cur = cur->next;
+		add_ra_option_abro(cur->sb, iface->AdvAbroList);
 	}
+
+	// Return the root of the list
+	return sbl;
 }
 
 static int send_ra(int sock, struct Interface *iface, struct in6_addr const *dest)
@@ -728,25 +807,124 @@ static int send_ra(int sock, struct Interface *iface, struct in6_addr const *des
 
 	update_iface_times(iface);
 
-	char address_text[INET6_ADDRSTRLEN] = { "" };
-	inet_ntop(AF_INET6, dest, address_text, INET6_ADDRSTRLEN);
-	dlog(LOG_DEBUG, 5, "sending RA to %s on %s", address_text, iface->props.name);
+	char dest_text[INET6_ADDRSTRLEN] = { "" };
+	char src_text[INET6_ADDRSTRLEN] = { "" };
+	addrtostr(dest, dest_text, INET6_ADDRSTRLEN);
+	addrtostr(iface->props.if_addr_rasrc, src_text, INET6_ADDRSTRLEN);
 
-	struct safe_buffer safe_buffer = SAFE_BUFFER_INIT;
+	// Build RA header
+	struct safe_buffer *ra_hdr = new_safe_buffer();
+	add_ra_header(ra_hdr, &iface->ra_header_info, iface->state_info.cease_adv);
+	// Build RA option list
+	struct safe_buffer_list *ra_opts = build_ra_options(iface, dest);
 
-	build_ra(&safe_buffer, iface);
+	/* *INDENT-OFF* */
+	/*
+	 *	RFC4861: 6.2.3.  Router Advertisement Message Content
+	 *	   If including all options causes the size of an advertisement to
+	 *	   exceed the link MTU, multiple advertisements can be sent, each
+	 *	   containing a subset of the options.
+	 *
+	 *	RFC6980: 5.  Specification
+	 *		Nodes MUST NOT employ IPv6 fragmentation for sending any of the
+	 *		following Neighbor Discovery and SEcure Neighbor Discovery messages:
+	 *
+	 *		o  Neighbor Solicitation
+	 *		o  Neighbor Advertisement
+	 *		o  Router Solicitation
+	 *		o  Router Advertisement
+	 *		o  Redirect
+	 *		o  Certification Path Solicitation
+	 *
+	 *		Nodes SHOULD NOT employ IPv6 fragmentation for sending the following
+	 *		messages (see Section 6.4.2 of [RFC3971]):
+	 *
+	 *		o  Certification Path Advertisement
+	 *
+	*/
+	/* *INDENT-ON* */
 
-	int err = really_send(sock, dest, &iface->props, &safe_buffer);
+	// Send out one or more RAs, all in the form of (hdr+options),
+	// such that none of the RAs exceed the link MTU
+	// (max size is pre-computed in iface->props.max_ra_option_size)
 
-	safe_buffer_free(&safe_buffer);
+	struct safe_buffer_list *cur = ra_opts;
+	struct safe_buffer *sb = new_safe_buffer();
+	unsigned long int total_seen_options = 0;
+	do {
+		unsigned long int option_count = 0;
+		sb->used = 0;
+		// Duplicate the RA header
+		safe_buffer_append(sb, ra_hdr->buffer, ra_hdr->used);
+		// Copy in as many RA options as we can fit.
+		while(NULL != cur) {
+			if(sb->used == 0) {
+				dlog(LOG_DEBUG, 5, "send_ra: Saw empty buffer!");
+				cur = cur->next;
+				continue;
+			}
+			// Ok, it's more than 0 bytes in length
+			total_seen_options++;
+			// Not enough room for the next option in our buffer, just send the buffer now.
+			if(sb->used + cur->sb->used > iface->props.max_ra_option_size) {
+				// But make sure we send at least one option in each RA
+				// TODO: a future improvement would be to optimize packing of
+				// the options in the minimal number of RAs, such that each one
+				// does not exceed the MTU where possible.
+				if(option_count > 0)
+					break;
+			}
+			// It's possible that a single option is larger than the MTU, so
+			// fragmentation will always happen in that case.
+			// One known case is a very long DNSSL, which is documented in
+			// RFC6106 errata #4864
+			// In this case, the RA will contain a single option, consisting of
+			// ONLY the DNSSL, without other options. RFC6980-conforming nodes
+			// should then ignore the DNSSL.
+			if(cur->sb->used > iface->props.max_ra_option_size) {
+				flog(LOG_WARNING,
+						"send_ra: RA option (type=%hhd) too long for MTU, fragmenting anyway (violates RFC6980)",
+						(unsigned char)(cur->sb->buffer[0]));
+			}
+			// Add this option to the buffer.
+			safe_buffer_append(sb, cur->sb->buffer, cur->sb->used);
+			option_count++;
+			/*
+			if(cur->sb->used > 0 && (unsigned char)(cur->sb->buffer[0]) == ND_OPT_DNSSL_INFORMATION) {
+				abort();
+			}
+			*/
+			cur = cur->next;
+		}
 
-	if (err < 0) {
-		if (!iface->IgnoreIfMissing || !(errno == EINVAL || errno == ENODEV))
-			flog(LOG_WARNING, "sendmsg: %s", strerror(errno));
-		else
-			dlog(LOG_DEBUG, 3, "sendmsg: %s", strerror(errno));
-		return -1;
-	}
+		if(option_count == 0 && total_seen_options > 0) {
+			// If option_count == 0 and total_seen_options==0 we make sure to
+			// send ONE RA out, so that clients get the RA header fields.
+		} else if(option_count == 0 && total_seen_options > 0) {
+			// None of the RA options are scheduled for this window.
+			dlog(LOG_DEBUG, 5, "No RA options scheduled in this pass, staying quiet; already sent at least one RA packet");
+			break;
+		}
+
+		// RA built, now send it.
+		dlog(LOG_DEBUG, 5, "sending RA to %s on %s (%s), %lu options (using %lu/%u bytes)", dest_text, iface->props.name, src_text, option_count, sb->used, iface->props.max_ra_option_size);
+		int err = really_send(sock, dest, &iface->props, sb);
+		if (err < 0) {
+			if (!iface->IgnoreIfMissing || !(errno == EINVAL || errno == ENODEV))
+				flog(LOG_WARNING, "sendmsg: %s", strerror(errno));
+			else
+				dlog(LOG_DEBUG, 3, "sendmsg: %s", strerror(errno));
+			safe_buffer_free(sb);
+			safe_buffer_list_free(ra_opts);
+			safe_buffer_free(ra_hdr);
+			return -1;
+		}
+
+	} while(NULL != cur);
+
+	safe_buffer_free(sb);
+	safe_buffer_list_free(ra_opts);
+	safe_buffer_free(ra_hdr);
 
 	return 0;
 }
@@ -773,7 +951,7 @@ static int really_send(int sock, struct in6_addr const *dest, struct properties
 
 	struct in6_pktinfo *pkt_info = (struct in6_pktinfo *)CMSG_DATA(cmsg);
 	pkt_info->ipi6_ifindex = props->if_index;
-	memcpy(&pkt_info->ipi6_addr, &props->if_addr, sizeof(struct in6_addr));
+	memcpy(&pkt_info->ipi6_addr, props->if_addr_rasrc, sizeof(struct in6_addr));
 
 #ifdef HAVE_SIN6_SCOPE_ID
 	if (IN6_IS_ADDR_LINKLOCAL(&addr.sin6_addr) || IN6_IS_ADDR_MC_LINKLOCAL(&addr.sin6_addr))
@@ -791,3 +969,70 @@ static int really_send(int sock, struct in6_addr const *dest, struct properties
 
 	return sendmsg(sock, &mhdr, 0);
 }
+
+
+static int schedule_option_prefix(struct in6_addr const *dest, struct Interface const *iface, struct AdvPrefix const *prefix)
+{
+	return schedule_helper(dest, iface, prefix->curr_preferredlft);
+}
+
+static int schedule_option_route(struct in6_addr const *dest, struct Interface const *iface, struct AdvRoute const *route)
+{
+	return schedule_helper(dest, iface, route->AdvRouteLifetime);
+}
+
+static int schedule_option_rdnss(struct in6_addr const *dest, struct Interface const *iface, struct AdvRDNSS const *rdnss)
+{
+	return schedule_helper(dest, iface, rdnss->AdvRDNSSLifetime);
+}
+
+static int schedule_option_dnssl(struct in6_addr const *dest, struct Interface const *iface, struct AdvDNSSL const *dnssl)
+{
+	return schedule_helper(dest, iface, dnssl->AdvDNSSLLifetime);
+}
+
+static int schedule_option_mtu(struct in6_addr const *dest, struct Interface const *iface)
+{
+	return schedule_helper(dest, iface, iface->ra_header_info.AdvDefaultLifetime);
+}
+
+static int schedule_option_sllao(struct in6_addr const *dest, struct Interface const *iface)
+{
+	return schedule_helper(dest, iface, iface->ra_header_info.AdvDefaultLifetime);
+}
+
+static int schedule_option_mipv6_rtr_adv_interval(struct in6_addr const *dest, struct Interface const *iface)
+{
+	return schedule_helper(dest, iface, iface->ra_header_info.AdvDefaultLifetime);
+}
+
+static int schedule_option_mipv6_home_agent_info(struct in6_addr const *dest, struct Interface const *iface)
+{
+	return schedule_helper(dest, iface, iface->mipv6.HomeAgentLifetime);
+}
+
+static int schedule_option_lowpanco(struct in6_addr const *dest, struct Interface const *iface)
+{
+	return schedule_helper(dest, iface, iface->AdvLowpanCoList->AdvLifeTime);
+}
+
+static int schedule_option_abro(struct in6_addr const *dest, struct Interface const *iface)
+{
+	return schedule_helper(dest, iface, iface->AdvAbroList->ValidLifeTime);
+}
+
+static int schedule_helper(struct in6_addr const *dest, struct Interface const *iface, int option_lifetime)
+{
+	return 1;
+	// 1.
+	// Cases to schedule a complete RA blast:
+	// - Server received a RS
+	// - We're in the initial-RAs phase of startup
+	// - The (unicast) destination was first seen very recently, and probably
+	//   has NOT got a full set of RAs yet.
+
+	// 2.
+	// If the dest has existed for a while
+	// (unicast destination): spread RAs out to at least 1/N of the option lifetime
+	// (multicast destination): spread RAs out to at least 1/N of the option lifetime
+};
