@@ -39,38 +39,119 @@ void safe_buffer_free(struct safe_buffer * sb)
 	}
 }
 
+/**
+ * Requests that the safe_buffer capacity be least n bytes in size.
+ *
+ * If n is greater than the current capacity, the function causes the container
+ * to reallocate its storage increasing its capacity to n (or greater).
+ *
+ * In all other cases, the function call does not cause a reallocation and the
+ * capacity is not affected.
+ *
+ * @param sb safe_buffer to enlarge
+ * @param b Minimum capacity for the safe_buffer.
+ */
+void safe_buffer_resize(struct safe_buffer * sb, size_t n)
+{
+	const int blocksize = 1 << 6; // MUST BE POWER OF 2.
+	if (sb->allocated < n) {
+		if(n % blocksize > 0) {
+			n |= (blocksize-1); // Set all the low bits
+			n++;
+		}
+		if (n > 64*1024) {
+			flog(LOG_ERR, "Requested buffer too large for any possible IPv6 ND, even with jumbogram.  Exiting.");
+			exit(1);
+		}
+		sb->allocated = n;
+		sb->buffer = realloc(sb->buffer, sb->allocated);
+	}
+}
+
 size_t safe_buffer_pad(struct safe_buffer * sb, size_t count)
 {
-	size_t rc = 0;
-	unsigned char zero = 0;
-
-	while (count--) {
-		rc += safe_buffer_append(sb, &zero, 1);
-	}
-
-	return rc;
+	safe_buffer_resize(sb, sb->used + count);
+	memset(&sb->buffer[sb->used], (uint8_t)0, count);
+	sb->used += count;
+	return count;
 }
 
 size_t safe_buffer_append(struct safe_buffer * sb, void const * v, size_t count)
 {
 	if (sb) {
 		unsigned const char * m = (unsigned const char *)v;
-		if (sb->allocated <= sb->used + count) {
-			sb->allocated = sb->used + count + MSG_SIZE_SEND;
-			sb->buffer = realloc(sb->buffer, sb->allocated);
-		}
+		safe_buffer_resize(sb, sb->used + count);
 		memcpy(&sb->buffer[sb->used], m, count);
 		sb->used += count;
-
-		if (sb->used >= MSG_SIZE_SEND) {
-			flog(LOG_ERR, "Too many prefixes, routes, rdnss or dnssl to fit in buffer.  Exiting.");
-			exit(1);
-		}
 	}
 
 	return count;
 }
 
+/**
+ * Create a new safe_buffer_list
+ *
+ * @return new safe_buffer_list, with a safe_buffer on the heap.
+ */
+struct safe_buffer_list * new_safe_buffer_list(void)
+{
+	struct safe_buffer_list * sbl = malloc(sizeof(struct safe_buffer_list));
+	sbl->sb = new_safe_buffer();
+	sbl->next = NULL;
+	return sbl;
+}
+
+/**
+ * Ensure list tail has an empty buffer ready to accept data.
+ *
+ * If the present element is empty of data, just return it.
+ * Otherwise return a new safe_buffer_list ready to accept data.
+ *
+ * @param sbl safe_buffer_list.
+ * @return new tail of list.
+ */
+struct safe_buffer_list * safe_buffer_list_append(struct safe_buffer_list * sbl)
+{
+	// Only allocate a new entry if this one has bytes in it.
+	if(sbl->sb && sbl->sb->used > 0) {
+		struct safe_buffer_list * next = new_safe_buffer_list();
+		sbl->next = next;
+		sbl = next;
+	}
+	return sbl;
+}
+
+/**
+ * Convert an entire safe_buffer_list to a single safe_buffer.
+ *
+ * @param sbl safe_buffer_list source.
+ * @param sb  safe_buffer destination.
+ */
+void safe_buffer_list_to_safe_buffer(struct safe_buffer_list * sbl, struct safe_buffer *sb)
+{
+	struct safe_buffer_list *cur;
+	for(cur = sbl; cur; cur = cur->next) {
+		if(cur->sb)
+			safe_buffer_append(sb, cur->sb->buffer, cur->sb->used);
+	}
+}
+
+/**
+ * Free all memory used by a safe_buffer_list
+ *
+ * @param sbl safe_buffer_list to free.
+ */
+void safe_buffer_list_free(struct safe_buffer_list * sbl)
+{
+	struct safe_buffer_list * next;
+	for (struct safe_buffer_list *current = sbl; current; current = next) {
+		if(current->sb)
+			safe_buffer_free(current->sb);
+		next = current->next;
+		free(current);
+	}
+}
+
 __attribute__ ((format(printf, 1, 2)))
 char * strdupf(char const * format, ...)
 {
@@ -93,11 +174,11 @@ double rand_between(double lower, double upper)
 }
 
 /* This assumes that str is not null and str_size > 0 */
-void addrtostr(struct in6_addr *addr, char *str, size_t str_size)
+void addrtostr(struct in6_addr const *addr, char *str, size_t str_size)
 {
 	const char *res;
 
-	res = inet_ntop(AF_INET6, (void *)addr, str, str_size);
+	res = inet_ntop(AF_INET6, (void const*)addr, str, str_size);
 
 	if (res == NULL) {
 		flog(LOG_ERR, "addrtostr: inet_ntop: %s", strerror(errno));
